##-----------------------------------------------------
## Design       : medianfilter_pt_ref
## File Name    : medianfilter_pt_ref.tcl
## Purpose      : PrimeTime STA & PA reference script for medianfilter 
## Limitation   : none
## Errors       : none known
## Include Files: none
## Author       : Liang Liu, liang.liu@eit.lth.se, Lund University
## Software     : Synopsys PrimeTime
##---------------------------------------------------
## Revision List:
##+-----------+--------------------+-----------------+----------------------------------+
##| Version   |Author              | Date            | Changes	                        |
##+-----------+--------------------+-----------------+----------------------------------+
##|1.0        |       Liang Liu    | 2013/02/15      | original created                 |
##+-----------+--------------------+-----------------+----------------------------------+ 


##################  remove any previous designs 
remove_design -all

################### set up power analysis mode #####################
# step 1: enalbe power analysis and set analysis mode 

set power_enable_analysis  true
set power_analysis_mode time_based


####################### set up libaries ############################
# step 2: link to your design libary 
 
set search_path "$env(FAR_LIB) $search_path"

set link_library "* fsc0l_d_generic_core_ff1p32vm40c.db \
		  fsc0l_d_generic_core_ss1p08v125c.db \
		  fsc0l_d_generic_core_tt1p2v25c.db \
		  foc0l_a33_t33_generic_io_ff1p32vm40c.db \
		  foc0l_a33_t33_generic_io_ss1p08v125c.db \
		  foc0l_a33_t33_generic_io_tt1p2v25c.db"

set target_library "fsc0l_d_generic_core_ff1p32vm40c.db \
                    fsc0l_d_generic_core_ss1p08v125c.db \
                    fsc0l_d_generic_core_tt1p2v25c.db \
		                foc0l_a33_t33_generic_io_ff1p32vm40c.db \
                    foc0l_a33_t33_generic_io_ss1p08v125c.db \
                    foc0l_a33_t33_generic_io_tt1p2v25c.db"
                    

set symbol_library "fsc0l_d_generic_core.sdb foc0l_a33_t33_generic_io.sdb"

set synthetic_library "standard.sldb dw_foundation.sldb"

set link_library "$link_library ./SHLD130_128X32X1BM1_TC.db"
set target_library "$target_library ./SHLD130_128X32X1BM1_TC.db"
set link_library "$link_library ./SPLD130_512X14BM1A_TC.db"
set target_library "$target_library ./SPLD130_512X14BM1A_TC.db"
 

 
####################### design input    ############################
# step 3: read your design (netlist) & link design

read_verilog ./netlists/pnr_netlist.v
current_design MEDIANFILTER_N8
link


####################### timing constraint ##########################
# step 4: setup timing constraint (or read sdc file)

read_sdc ./netlists/pnr_medfilt.sdc





####################### Back annotate     ##########################
# step 5: back annotate delay information (read sdf file)

read_parasitics ./netlists/MEDIANFILTER_N8_place.spef
read_sdf -type sdf_max ./netlists/pnr_netlist.sdf



################# read switching activity file #####################
# step 6: read vcd file obtained from post-layout (syn) simulation

read_vcd -strip_path medianfilter_tb/dut ./netlists/pnr_medfilt.vcd


####################### analysis and report #################
# step 6: output report

check_power
update_power
report_power -verbose > ./report/power_medfilter.rpt

report_timing -delay_type min -max_paths 10 > ./report/timing_hold_medfilter.rpt
report_timing -delay_type max -max_paths 10 > ./report/timing_setup_medfilter.rpt
report_clock_timing -type skew -verbose > ./report/timing_clk_medfilter.rpt

